/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2009  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_R32C.c for RENESAS R32C with HEW compiler
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                   /* Main clock frequency         */
  #define OS_FSYS 16000000uL        /* External oscillator          */
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER OS_FSYS     /* We use external clock        */
#endif

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART OS_FSYS      /* We use external clock        */
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*       and supports UART0 and UART1
*/
#ifndef   OS_UART
  #define OS_UART 0
#endif
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400uL
#endif

/****** End of configurable options *********************************/

#define OS_UART_USED (OS_VIEW_ENABLE && ((OS_UART == 0) || (OS_UART == 1)))

/*********************************************************************
*
*       SFRs used in Rtosinit
*
**********************************************************************
*/

/******	Timer A0 ****************************************************/

#define TA0     (*(volatile OS_U16*) (0x00000346))
#define TA0MR   (*(volatile OS_U8*)  (0x00000356))
#define TA0IC   (*(volatile OS_U8*)  (0x0000006C))
#define TABSR   (*(volatile OS_U8*)  (0x00000340))

#define TCSPR   (*(volatile OS_U8*)  (0x00000035F))


/******	Protection / Processor mode**********************************/

#define PRR     (*(volatile OS_U8*)  (0x000000007))
#define PRCR    (*(volatile OS_U8*)  (0x00004004A))
#define PRCR2   (*(volatile OS_U8*)  (0x00004405F))
#define PM2     (*(volatile OS_U8*)  (0x000040053))
#define PM3     (*(volatile OS_U8*)  (0x000040048))

/****** Clock control ***********************************************/

#define CCR     (*(volatile OS_U8*)  (0x000000004))
#define CM0     (*(volatile OS_U8*)  (0x000040046))
#define CM1     (*(volatile OS_U8*)  (0x000040047))
#define CM2     (*(volatile OS_U8*)  (0x00004004D))
#define CM3     (*(volatile OS_U8*)  (0x00004005A))
#define CPSRF   (*(volatile OS_U8*)  (0x000000341))

/****** PLL *********************************************************/

#define PLC0    (*(volatile OS_U8*)  (0x000040020))
#define PLC1    (*(volatile OS_U8*)  (0x000040021))

/****** Peripheral Control ******************************************/

#define PBC     (*(volatile OS_U16*) (0x00000001E))

/******	UART ********************************************************/

#define U0TB    (*(volatile OS_U8*)  (0x0000036A))
#define U0RB    (*(volatile OS_U16*) (0x0000036E))
#define U0C1    (*(volatile OS_U8*)  (0x0000036D))
#define U0MR    (*(volatile OS_U8*)  (0x00000368))
#define U0C0    (*(volatile OS_U8*)  (0x0000036C))
#define U0BRG   (*(volatile OS_U8*)  (0x00000369))
#define S0RIC   (*(volatile OS_U8*)  (0x00000072))
#define S0TIC   (*(volatile OS_U8*)  (0x00000090))
#define P6_3S   (*(volatile OS_U8*)  (0x000400D6))

#define U1TB    (*(volatile OS_U8*)  (0x000002EA))
#define U1RB    (*(volatile OS_U16*) (0x000002EE))
#define U1C1    (*(volatile OS_U8*)  (0x000002ED))
#define U1MR    (*(volatile OS_U8*)  (0x000002E8))
#define U1C0    (*(volatile OS_U8*)  (0x000002EC))
#define U1BRG   (*(volatile OS_U8*)  (0x000002E9))
#define S1RIC   (*(volatile OS_U8*)  (0x00000074))
#define S1TIC   (*(volatile OS_U8*)  (0x00000092))
#define P6_7S   (*(volatile OS_U8*)  (0x000400DE))

#if OS_UART_USED
  // Assigne SFRs and interrupt vectors for UART
  #if (OS_UART == 0)
    #define UARTX_RX_VECT (18)
    #define UARTX_TX_VECT (17)
    #define UxTB          (U0TB)
    #define UxRB          (U0RB)
    #define UxC1          (U0C1)
    #define UxMR          (U0MR)
    #define UxC0          (U0C0)
    #define UxBRG         (U0BRG)
    #define UxC1          (U0C1)
    #define SxRIC         (S0RIC)
    #define SxTIC         (S0TIC)
    #define TX_PORT_BIT   (3)
    #define UART_PORT_PS  (P6_3S)
  #elif (OS_UART == 1)
    #define UARTX_RX_VECT (20)
    #define UARTX_TX_VECT (19)
    #define UxTB          (U1TB)
    #define UxRB          (U1RB)
    #define UxC1          (U1C1)
    #define UxMR          (U1MR)
    #define UxC0          (U1C0)
    #define UxBRG         (U1BRG)
    #define UxC1          (U1C1)
    #define SxRIC         (S1RIC)
    #define SxTIC         (S1TIC)
    #define TX_PORT_BIT   (7)
    #define UART_PORT_PS  (P6_7S)
  #endif
#endif  // OS_UART_USED

/******	Ports *******************************************************/

#define PD6     (*(volatile OS_U8*)  (0x000003CE))
#define P6      (*(volatile OS_U8*)  (0x000003CC))

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  unsigned int n;
  //
  // Setup CPU clock oscillator and PLL
  //
  // - external clock    16 MHz
  // - PLL clock        128 MHz
  // - CPU clock         64 MHz
  // - Base clock        64 MHz
  // - Peripheral clock  32 MHz
  PRCR  = 0xFF;       // Enable write to control registers
  PRCR2 = 0x80;       // Enable write to CM3 register
  PRR   = 0xAA;       // Enable write to CCR, FMCR, PBC register

  PM2   = 0x02;       // Processor mode register 2: enable clock change
  CM0   = 0x08;       // System clock control register 0
  CM1   = 0x20;       // System clock control register 1
  CM2   = 0x00;       // Oscillation stop detect register
  CM3   = 0x00;       // Low speed mode clock control register
  TCSPR = 0x00;       // Count source prescaler register
  CPSRF = 0x00;       // Clock prescaler reset register
  PBC   = 0x0504;     // Peripheral bus control register
                      // Value depends on setting of CCR register
  CCR   = 0x1F;       // Clock control register
                      // 11: Base clock divide ratio:       /2
                      // 11: CPU clock divide ratio:        /1
                      // 01: Peripheral clock divide ratio: /2
  PRCR  = 0xFF;
  PLC0  = 0x45;       // Pll control register 0: 01000101
  PRCR  = 0xFF;
  PLC1  = 0x03;       // Pll control register 1
  PLC1 &= ~(1 << 4);  // SEO = 0 => PLL mode
  //
  // Small delay
  //
  for (n = 0; n < 0x8000; n++) {
    _asm("NOP");
  }
  PRR   = 0xAA;       // Enable write to CCR, FMCR, PBC register
  CCR  &= ~(1 << 7);  // Base clock source is PLL

  PRCR  = 0xFF;
  PM3   = 0x40;       // Peripheral clock = PLL/4
  PM2   = 0x40;       // Processor mode register 2: disable clock change
                      // Count source for f2n is main clock
  PRR   = 0x00;       // Disable write to CCR, FMCR, PBC register
  //
  // Ensure external clock (f2n) is enabled as source for timer (and UART)
  //
  PRCR  = (0 << 0)         // PRC0: 1 => Enable write to CM0, CM1, CM2, PM3
        | (1 << 1)         // PRC1: 1 => Enable write to PM0, PM2, INVC0, INVC1 register
        | (0 << 2)         // PRC2: 1 => Enable write to PD9, P9_iS, PLC0, PLC1
        ;
  PM2  |= (1 << 6);        // PM26: 1 => select Xin as f2n clock source
  TCSPR = 0                // Divider for f2n: 0 => do not divide, divide by 1
        | (1 << 7)         // CST: 1 => Enable f2n clock
        ;
  PRCR  = 0x00;            // Disable write to control registers
  //
  // Initialize timer for embOS
  //
  TA0   = (OS_PCLK_TIMER/1000 - 1);  // Generate Interrupts every 1 ms
  TA0MR = 0x80;            // Timer mode register
                           // bit 01 : 00 Timer mode
                           // bit 2  :  0 No puls output
                           // bit 34 : 0X No gate function
                           // bit 5  : 0
                           // bit 67 : 01 clock source : f/2n
  TA0IC  = 0x01;           // Timer interrupt control register, reset pending, priority 1 (lowest)
  TABSR |= 1;              // Set Timer count start flag
  OS_COM_Init();           // Initialize UART for embOS Viewer
}

/*********************************************************************
*
*       OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to exec
  for (;;) {            // Nothing to do ... wait for interrupt
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via OSView or high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  unsigned int T_cnt;
  OS_U32 Time;

  T_cnt = TA0;
  Time = OS_Time + 1;
  if (TA0IC & (1 << 3)) {   // If timer interrupt pending ...
    T_cnt = TA0;            // adjust result
    Time++;
  }
  return (OS_PCLK_TIMER/1000) * Time - T_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick()
*       The OS system timer interrupt
*/
#pragma INTERRUPT OS_ISR_Tick (vect=12);
void OS_ISR_Tick (void);
void OS_ISR_Tick (void) {
  OS_EnterNestableInterrupt();
  OS_TICK_Handle();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView
*
**********************************************************************
*/
#if OS_UART_USED
#define BAUDDIVIDE (((OS_PCLK_UART + (OS_BAUDRATE*8L)) / (OS_BAUDRATE*16L)) - 1)

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
void OS_COM_Send1(OS_U8 c) {
  UxTB = c;
}

/*********************************************************************
*
*       OS_ISR_rx()
*       OS UART rx interrupt handler
*/
#if (OS_UART == 0)
#pragma INTERRUPT OS_ISR_rx (vect=18);
#elif (OS_UART == 1)
#pragma INTERRUPT OS_ISR_rx (vect=20);
#endif
void OS_ISR_rx (void);
void OS_ISR_rx (void) {
  int Data;
  OS_EnterNestableInterrupt();  /*  We will enable interrupts */
  Data = UxRB;
  if (Data & 0x6000) {   /* Check if errors occurred */
    UxC1 &= ~(1 << 2);   /* disable Rx               */
    UxC1 |=  (1 << 2);   /* enable Rx                */
  } else {
    OS_OnRx(Data);
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_tx()
*       OS UART tx interrupt handler
*/
#if (OS_UART == 0)
#pragma INTERRUPT OS_ISR_tx (vect=17);
#elif (OS_UART == 1)
#pragma INTERRUPT OS_ISR_tx (vect=19);
#endif
void OS_ISR_tx (void);
void OS_ISR_tx (void) {
  OS_EnterNestableInterrupt();
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/
void OS_COM_Init(void) {
  UxMR   = 0x00;               //  Lock Sio, error reset
  UxC0   = 0x02                //  Use f2n as clock source
         | (0 << 2)            //  Reserved, always write 0
         | (0 << 3)            //  TxEPT, RO, write 0
         | (1 << 4)            //  CRD:   1 => Disable CTS
         | (0 << 5)            //  NCH:   1 => TxD Output is push-pull
         | (0 << 6)            //  CKPOL: don't care for UART mode, write 0
         | (0 << 7)            //  UFORM: 0 => LSB first
         ;
  UxBRG  = BAUDDIVIDE;         //  Calculated Baudrate
  UxC1   = 0;                  //  Lock Rx and Tx
  UxMR   = 0x05                //  8  Data
         | (0 << 4)            //  0: 1 Stop-Bit (1= 2 Stop-Bits)
         | (0 << 5)            //  X: Parity select
         | (0 << 6)            //  1: parity enable
         | (0 << 7)            //  0: no sleep
		 ;
  UxC1   = 5;                  //  enable reception and transmission
  SxRIC  = 1;                  //  Enable UART Rx interrupts
  SxTIC  = 1;                  //  Enable UART Tx interrupts
  PD6   |= (1 << TX_PORT_BIT); //  Direction register   (0=IN 1=OUT)
  P6    |= (1 << TX_PORT_BIT); //  Register  (0=Low, 1= High)
  UART_PORT_PS = 3;            //  Enable TxD function
}

#else  /* No communication routines */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy  */
  }
  void OS_Sendstring(void) {
  }
#endif // OS_UART_USED

/****** End Of File *************************************************/

